/********************************************************************************
** Form generated from reading UI file 'arenaselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ARENASELECTOR_H
#define UI_ARENASELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ArenaSelectorBase
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout;
    QListWidget *arenaList;
    QCheckBox *kcfg_RandomArenaMode;
    QPushButton *getNewButton;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *arenaDescription;
    QLabel *arenaAuthor;
    QLabel *arenaContact;
    QGraphicsView *arenaPreview;
    QLineEdit *kcfg_Arena;

    void setupUi(QWidget *ArenaSelectorBase)
    {
        if (ArenaSelectorBase->objectName().isEmpty())
            ArenaSelectorBase->setObjectName("ArenaSelectorBase");
        ArenaSelectorBase->resize(520, 421);
        verticalLayout = new QVBoxLayout(ArenaSelectorBase);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setObjectName("vboxLayout");
        arenaList = new QListWidget(ArenaSelectorBase);
        arenaList->setObjectName("arenaList");

        vboxLayout->addWidget(arenaList);

        kcfg_RandomArenaMode = new QCheckBox(ArenaSelectorBase);
        kcfg_RandomArenaMode->setObjectName("kcfg_RandomArenaMode");

        vboxLayout->addWidget(kcfg_RandomArenaMode);

        getNewButton = new QPushButton(ArenaSelectorBase);
        getNewButton->setObjectName("getNewButton");

        vboxLayout->addWidget(getNewButton);


        hboxLayout->addLayout(vboxLayout);

        groupBox = new QGroupBox(ArenaSelectorBase);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        arenaDescription = new QLabel(groupBox);
        arenaDescription->setObjectName("arenaDescription");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(arenaDescription->sizePolicy().hasHeightForWidth());
        arenaDescription->setSizePolicy(sizePolicy1);
        arenaDescription->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        arenaDescription->setWordWrap(true);
        arenaDescription->setOpenExternalLinks(false);
        arenaDescription->setTextInteractionFlags(Qt::LinksAccessibleByMouse);

        gridLayout->addWidget(arenaDescription, 1, 1, 1, 1);

        arenaAuthor = new QLabel(groupBox);
        arenaAuthor->setObjectName("arenaAuthor");
        sizePolicy1.setHeightForWidth(arenaAuthor->sizePolicy().hasHeightForWidth());
        arenaAuthor->setSizePolicy(sizePolicy1);
        QFont font;
        font.setItalic(true);
        arenaAuthor->setFont(font);
        arenaAuthor->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        arenaAuthor->setWordWrap(true);
        arenaAuthor->setOpenExternalLinks(false);
        arenaAuthor->setTextInteractionFlags(Qt::LinksAccessibleByMouse);

        gridLayout->addWidget(arenaAuthor, 2, 1, 1, 1);

        arenaContact = new QLabel(groupBox);
        arenaContact->setObjectName("arenaContact");
        sizePolicy1.setHeightForWidth(arenaContact->sizePolicy().hasHeightForWidth());
        arenaContact->setSizePolicy(sizePolicy1);
        arenaContact->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        arenaContact->setWordWrap(true);
        arenaContact->setOpenExternalLinks(true);
        arenaContact->setTextInteractionFlags(Qt::LinksAccessibleByMouse);

        gridLayout->addWidget(arenaContact, 3, 1, 1, 1);

        arenaPreview = new QGraphicsView(groupBox);
        arenaPreview->setObjectName("arenaPreview");
        arenaPreview->setFrameShape(QFrame::NoFrame);

        gridLayout->addWidget(arenaPreview, 0, 1, 1, 1);


        hboxLayout->addWidget(groupBox);


        verticalLayout->addLayout(hboxLayout);

        kcfg_Arena = new QLineEdit(ArenaSelectorBase);
        kcfg_Arena->setObjectName("kcfg_Arena");
        kcfg_Arena->setEnabled(false);
        kcfg_Arena->setFrame(false);
        kcfg_Arena->setReadOnly(false);

        verticalLayout->addWidget(kcfg_Arena);


        retranslateUi(ArenaSelectorBase);

        QMetaObject::connectSlotsByName(ArenaSelectorBase);
    } // setupUi

    void retranslateUi(QWidget *ArenaSelectorBase)
    {
        kcfg_RandomArenaMode->setText(tr2i18n("Random Mode", nullptr));
        getNewButton->setText(tr2i18n("Download New Arenas\342\200\246", nullptr));
        arenaDescription->setText(QString());
        arenaAuthor->setText(QString());
        arenaContact->setText(QString());
        (void)ArenaSelectorBase;
    } // retranslateUi

};

namespace Ui {
    class ArenaSelectorBase: public Ui_ArenaSelectorBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // ARENASELECTOR_H

