/********************************************************************************
** Form generated from reading UI file 'generalsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GENERALSETTINGS_H
#define UI_GENERALSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_GeneralSettings
{
public:
    QGridLayout *gridLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QCheckBox *kcfg_ShowAllTiles;
    QLabel *label_9;
    QLabel *label_8;
    QSpinBox *kcfg_BlastTime3;
    QSpinBox *kcfg_BlastTime1;
    QSpinBox *kcfg_BlastTime2;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_7;
    QLabel *label_10;
    QSpinBox *kcfg_BlastTime5;
    QSpinBox *kcfg_BlastTime4;
    QLabel *label_11;
    QSpacerItem *verticalSpacer;
    QFormLayout *formLayout;
    QLabel *label_2;
    QSpinBox *kcfg_PointsToWin;
    QLabel *label_3;
    QSpinBox *kcfg_RoundTime;
    QLabel *label_4;
    QSpinBox *kcfg_InitialSpeed;
    QLabel *label_5;
    QSpinBox *kcfg_InitialBombPower;
    QLabel *label_6;
    QSpinBox *kcfg_InitialBombArmory;
    QSpacerItem *horizontalSpacer;
    QSpinBox *kcfg_Dummy;

    void setupUi(QWidget *GeneralSettings)
    {
        if (GeneralSettings->objectName().isEmpty())
            GeneralSettings->setObjectName("GeneralSettings");
        GeneralSettings->resize(389, 489);
        gridLayout = new QGridLayout(GeneralSettings);
        gridLayout->setObjectName("gridLayout");
        groupBox = new QGroupBox(GeneralSettings);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        kcfg_ShowAllTiles = new QCheckBox(groupBox);
        kcfg_ShowAllTiles->setObjectName("kcfg_ShowAllTiles");

        gridLayout_2->addWidget(kcfg_ShowAllTiles, 0, 0, 1, 1);

        label_9 = new QLabel(groupBox);
        label_9->setObjectName("label_9");

        gridLayout_2->addWidget(label_9, 2, 0, 1, 1);

        label_8 = new QLabel(groupBox);
        label_8->setObjectName("label_8");

        gridLayout_2->addWidget(label_8, 5, 0, 1, 1);

        kcfg_BlastTime3 = new QSpinBox(groupBox);
        kcfg_BlastTime3->setObjectName("kcfg_BlastTime3");
        kcfg_BlastTime3->setMaximum(10000);

        gridLayout_2->addWidget(kcfg_BlastTime3, 5, 2, 1, 1);

        kcfg_BlastTime1 = new QSpinBox(groupBox);
        kcfg_BlastTime1->setObjectName("kcfg_BlastTime1");
        kcfg_BlastTime1->setMaximum(10000);

        gridLayout_2->addWidget(kcfg_BlastTime1, 1, 2, 1, 1);

        kcfg_BlastTime2 = new QSpinBox(groupBox);
        kcfg_BlastTime2->setObjectName("kcfg_BlastTime2");
        kcfg_BlastTime2->setMaximum(10000);

        gridLayout_2->addWidget(kcfg_BlastTime2, 2, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 2, 3, 1, 1);

        label_7 = new QLabel(groupBox);
        label_7->setObjectName("label_7");

        gridLayout_2->addWidget(label_7, 1, 0, 1, 2);

        label_10 = new QLabel(groupBox);
        label_10->setObjectName("label_10");

        gridLayout_2->addWidget(label_10, 8, 0, 1, 1);

        kcfg_BlastTime5 = new QSpinBox(groupBox);
        kcfg_BlastTime5->setObjectName("kcfg_BlastTime5");
        kcfg_BlastTime5->setMaximum(10000);

        gridLayout_2->addWidget(kcfg_BlastTime5, 8, 2, 1, 1);

        kcfg_BlastTime4 = new QSpinBox(groupBox);
        kcfg_BlastTime4->setObjectName("kcfg_BlastTime4");
        kcfg_BlastTime4->setMaximum(10000);

        gridLayout_2->addWidget(kcfg_BlastTime4, 6, 2, 1, 1);

        label_11 = new QLabel(groupBox);
        label_11->setObjectName("label_11");

        gridLayout_2->addWidget(label_11, 6, 0, 1, 1);


        gridLayout->addWidget(groupBox, 1, 0, 1, 2);

        verticalSpacer = new QSpacerItem(377, 62, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 3, 0, 1, 2);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label_2 = new QLabel(GeneralSettings);
        label_2->setObjectName("label_2");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_2);

        kcfg_PointsToWin = new QSpinBox(GeneralSettings);
        kcfg_PointsToWin->setObjectName("kcfg_PointsToWin");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(kcfg_PointsToWin->sizePolicy().hasHeightForWidth());
        kcfg_PointsToWin->setSizePolicy(sizePolicy);
        kcfg_PointsToWin->setMinimum(1);
        kcfg_PointsToWin->setMaximum(20);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, kcfg_PointsToWin);

        label_3 = new QLabel(GeneralSettings);
        label_3->setObjectName("label_3");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_3);

        kcfg_RoundTime = new QSpinBox(GeneralSettings);
        kcfg_RoundTime->setObjectName("kcfg_RoundTime");
        sizePolicy.setHeightForWidth(kcfg_RoundTime->sizePolicy().hasHeightForWidth());
        kcfg_RoundTime->setSizePolicy(sizePolicy);
        kcfg_RoundTime->setMinimum(10);
        kcfg_RoundTime->setMaximum(600);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, kcfg_RoundTime);

        label_4 = new QLabel(GeneralSettings);
        label_4->setObjectName("label_4");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_4);

        kcfg_InitialSpeed = new QSpinBox(GeneralSettings);
        kcfg_InitialSpeed->setObjectName("kcfg_InitialSpeed");
        sizePolicy.setHeightForWidth(kcfg_InitialSpeed->sizePolicy().hasHeightForWidth());
        kcfg_InitialSpeed->setSizePolicy(sizePolicy);
        kcfg_InitialSpeed->setMinimum(2);
        kcfg_InitialSpeed->setMaximum(10);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, kcfg_InitialSpeed);

        label_5 = new QLabel(GeneralSettings);
        label_5->setObjectName("label_5");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, label_5);

        kcfg_InitialBombPower = new QSpinBox(GeneralSettings);
        kcfg_InitialBombPower->setObjectName("kcfg_InitialBombPower");
        sizePolicy.setHeightForWidth(kcfg_InitialBombPower->sizePolicy().hasHeightForWidth());
        kcfg_InitialBombPower->setSizePolicy(sizePolicy);
        kcfg_InitialBombPower->setMinimum(1);
        kcfg_InitialBombPower->setMaximum(10);

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, kcfg_InitialBombPower);

        label_6 = new QLabel(GeneralSettings);
        label_6->setObjectName("label_6");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, label_6);

        kcfg_InitialBombArmory = new QSpinBox(GeneralSettings);
        kcfg_InitialBombArmory->setObjectName("kcfg_InitialBombArmory");
        sizePolicy.setHeightForWidth(kcfg_InitialBombArmory->sizePolicy().hasHeightForWidth());
        kcfg_InitialBombArmory->setSizePolicy(sizePolicy);
        kcfg_InitialBombArmory->setMinimum(1);
        kcfg_InitialBombArmory->setMaximum(10);

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, kcfg_InitialBombArmory);


        gridLayout->addLayout(formLayout, 0, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 1, 1, 1);

        kcfg_Dummy = new QSpinBox(GeneralSettings);
        kcfg_Dummy->setObjectName("kcfg_Dummy");

        gridLayout->addWidget(kcfg_Dummy, 2, 0, 1, 2);


        retranslateUi(GeneralSettings);

        QMetaObject::connectSlotsByName(GeneralSettings);
    } // setupUi

    void retranslateUi(QWidget *GeneralSettings)
    {
        groupBox->setTitle(tr2i18n("Only for Artists! Won't be here at release", nullptr));
        kcfg_ShowAllTiles->setText(tr2i18n(" Show all tiles", nullptr));
        label_9->setText(tr2i18n("Blast Time 2", nullptr));
        label_8->setText(tr2i18n("Blast Time 3", nullptr));
        label_7->setText(tr2i18n("Blast Time 1", nullptr));
        label_10->setText(tr2i18n("Blast Time 5", nullptr));
        label_11->setText(tr2i18n("Blast Time 4", nullptr));
        label_2->setText(tr2i18n("Points to win the game:", nullptr));
        label_3->setText(tr2i18n("Round Time in Seconds:", nullptr));
        label_4->setText(tr2i18n("Initial Speed:", nullptr));
        label_5->setText(tr2i18n("Initial Bomb Power:", nullptr));
        label_6->setText(tr2i18n("Initial Bomb Armory:", nullptr));
        (void)GeneralSettings;
    } // retranslateUi

};

namespace Ui {
    class GeneralSettings: public Ui_GeneralSettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // GENERALSETTINGS_H

